import 'dart:io';
import 'dart:convert' show utf8, jsonDecode;

import 'package:bottom_line/json_tool.dart';

import 'package:keychain/src/keychain_exception.dart';
import 'package:keychain/src/keychain_light.dart';
import 'package:keychain/src/keychain_cache.dart';


abstract class Keychain extends KeychainLight {

  
  static String fromString(String string) {
    return KeychainLight.fromString(string);
  }
  
  static String fromEnv(String envVar, {bool allowNull: false}) {
    if (_cache.environment[envVar] != null || allowNull) return _cache.environment[envVar];
    var result = Platform.environment[envVar];
    if (result == null && !allowNull) throw KeychainExeception('The environment variable "$envVar" was not found.');
    _cache.environment[envVar] = result;
    return result;
  }

  static String fromJsonFile(String key, String filePath, {bool allowNull: false}) {
    var varId = key + '|' + filePath;
    if (_cache.fileVar[varId] != null) return _cache.fileVar[varId];
    if (_cache.fileMap[filePath] == null) {
      var targetPath = filePath.split(Platform.pathSeparator);
      var scriptPath = Platform.script.pathSegments;
      var index = scriptPath.indexOf(targetPath[0]);
      if (index == -1) {
        throw KeychainExeception('Json file target must relative to a directory on the script path.');
      }
      var pathCopy = List.of(scriptPath);
      pathCopy.replaceRange(index, pathCopy.length, targetPath);
      var u = Uri.file(Platform.pathSeparator + pathCopy.join(Platform.pathSeparator), windows: Platform.isWindows);
      var str = File.fromUri(u).readAsStringSync();
      _cache.fileMap[filePath] = jsonDecode(str);
    }
    var r = MapTool(_cache.fileMap[filePath]).find(key);
    if (r == null && !allowNull) {
      throw KeychainExeception('Key "$key" is null which is not allowed. To allow null, set allowNull: true');
    }
    _cache.fileVar[varId] = r;
    return r;
    
  }

  static final _cache = KeychainCache();
  static final environment = Platform.environment;

}